/////////////////////////////////////////////////////////////////////
// Class Creator Version 2.0.000 Copyrigth (C) Poul A. Costinsky 1994
///////////////////////////////////////////////////////////////////
// Header File VirtMatrix.h
// class CWizVirtMatrix
// Template class of Matrix nXm, where the memory is allocated
// from virtual memory.
// The memory is actually allocated only on first access (read or write).
//
// 25/04/1996 17:47                             Author: Poul
///////////////////////////////////////////////////////////////////

#ifndef __CWizVirtMatrix_H
#define __CWizVirtMatrix_H

#include "virt_mem.h"

/////////////////////////////////////////////////////////////////////////////
// class CWizVirtMatrix
template<class TYPE>
class CWizVirtMatrix
{
public:
// Helper class to access the row
	//=========================================================
	class Row
	{
	public:
	// Constructors:
		Row(TYPE* ptr, const int m)
			: m_ptr(ptr), m_Cols(m) {}
		Row(Row& r)
			: m_ptr(r.m_ptr), m_Cols(r.m_Cols) {}
	// Operations:
		TYPE& operator[](int i)
			{
			ASSERT(i >= 0 && i < m_Cols);
			return m_ptr[i];
			}
		const TYPE& operator[](int i) const
			{
			ASSERT(i >= 0 && i < m_Cols);
			return m_ptr[i];
			}
	private:
		TYPE* m_ptr;
		const int m_Cols;
	};
	//=========================================================
	class ConstRow : protected Row
	{
	public:
		ConstRow(const TYPE* ptr, const int m)
			: Row(const_cast<TYPE*>(ptr), m) {}
		ConstRow(const Row& r)
			: Row(const_cast<Row&>(r)) {}
		const TYPE& operator[](int i) const
			{ return Row::operator[](i); }
	};
	//=========================================================
	// Psewdoclass to fill by zero
	struct Zero {};
	//=========================================================
// Constructors:
	// Create matrix nXm:
	CWizVirtMatrix (const int n, const int m)
		:	m_nRows (n),
			m_nCols (m),
			m_nElems (n*m),
			m_cVirtualMemory (m_nElems*sizeof(TYPE))
			{}
	// Create matrix nXm and init by 0:
	CWizVirtMatrix (const int n, const int m, Zero z)
		:	m_nRows (n),
			m_nCols (m),
			m_nElems (n*m),
			m_cVirtualMemory (m_nElems*sizeof(TYPE))
			{ ZeroInit(); }
	// Create matrix nXm and init by "init":
	CWizVirtMatrix (const int n, const int m, const TYPE& init)
		:	m_nRows (n),
			m_nCols (m),
			m_nElems (n*m),
			m_cVirtualMemory (m_nElems*sizeof(TYPE))
			{ Fill(init); }
	// Copy constructor:
	CWizVirtMatrix (const CWizVirtMatrix& T)
		:	m_nRows (T.m_nRows),
			m_nCols (T.m_nCols),
			m_nElems (T.m_nElems),
			m_cVirtualMemory (T.m_cVirtualMemory.GetSize())
			{ Copy(T); }
// Destructor:
	~CWizVirtMatrix () { }
public:
// Operations:
	CWizVirtMatrix& operator=(const CWizVirtMatrix& T)
		{
		ASSERT(m_nRows == T.m_nRows);
		ASSERT(m_nCols == T.m_nCols);
		Copy(T);
		return *this;
		}
	// Fills by 0
	CWizVirtMatrix& operator=(Zero z)
		{
		ZeroInit();
		return *this;
		}
	// Fills by init value
	void Fill(const TYPE& init)
		{
		TYPE* p = Ptr();
		for(INDEX i = 0; i < m_nElems; i++)
			*p++ = init;
		}
	// Access operator : a(2,4) = 5.67
	TYPE& operator()(const int i, const int j)
		{ return Get(i,j); }
	// Test Access operator : if(a(2,4) <= 5.67)...
	const TYPE& operator()(const int i, const int j) const
		{ return (const_cast<CWizVirtMatrix*>(this))->Get(i,j); }
	TYPE& Get(const int i, const int j)
		{
		ASSERT(i >= 0 && i < m_nRows);
		ASSERT(j >= 0 && j < m_nCols);
		return *(Ptr(i*m_nCols + j));
		}
	// Row access operator: 
	// like : CWizVirtMatrix<int>::Row row(a[i]);
	// row[3] = 5;
	Row operator[](int i)
		{
		ASSERT(i >= 0 && i < m_nRows);
		return Row(Ptr(i*m_nCols), m_nCols);
		}
	// Row constant access operator: 
	// like : CWizVirtMatrix<int>::ConstRow row(a[i]);
	// if(row[3] <= 5) ...
	const ConstRow operator[](int i) const
		{
		ASSERT(i >= 0 && i < m_nRows);
		return ConstRow(Ptr(i*m_nCols), m_nCols);
		}
protected:
// Implementation:
	void Copy(const CWizVirtMatrix& T)
		{
		ASSERT(m_cVirtualMemory.GetSize() == T.m_cVirtualMemory.GetSize());
		memcpy(Ptr(), T.Ptr(), m_cVirtualMemory.GetSize());
		}
	void ZeroInit()
		{ memset (Ptr(), 0, m_cVirtualMemory.GetSize()); }

	TYPE*		Ptr()		{ return (TYPE *)m_cVirtualMemory.GetPtr(); }
	const TYPE* Ptr() const { return (const TYPE*)m_cVirtualMemory.GetPtr(); }
	TYPE*		Ptr(int i)		 { return (Ptr() + i); }
	const TYPE* Ptr(int i) const { return (Ptr() + i); }
protected:
// Members:
	const int	m_nRows;
	const int	m_nCols;
	const int	m_nElems;
	CWizVirtualMemory	m_cVirtualMemory;
};

/////////////////////////////////////////////////////////////////////////////
#endif // __CWizVirtMatrix_H


